
import numpy as  np
import matplotlib.pyplot as plt
from IPython.core.display import display, HTML

def css_styling():
    styles = open("./styles/custom.css", "r").read()
    return HTML(styles)
display(css_styling())

def plot(A, color="black", scale=1., **kwargs) :
    if type(A) in [tuple, list] and len(A) == 2:
        α, x = A
        α, x = α.view(-1).data.cpu().numpy(), x.data.cpu().numpy()
        plt.scatter( x[:,0], x[:,1], scale*16*16 * α*len(α), color, **kwargs )

    else :
        if A.dim() == 2 :
            a = A.detach().cpu().numpy()
            a = np.vstack( (a, .6*a[0]+.4*a[-1] ) )
            plt.plot( a[:,0], a[:,1], linewidth=2, color=color, **kwargs)
        else :
            for A_k in A :
                plot(A_k, color=color, **kwargs)

def plot_worms(A, color="black", scale=1., **kwargs) :
    if A.dim() == 2 :
        a = A.detach().cpu().numpy()
        plt.scatter( a[0,0], a[0,1], 16*scale, color, **kwargs)
        plt.scatter( a[:,0], a[:,1],   scale, color, **kwargs)
    else :
        for A_k in A :
            plot_worms(A_k, color=color, scale=scale, **kwargs)

from model import Model

def train_and_display(Model, source, target, **kwargs) :
    # Model: orbit of the source
    model = Model( source ) 
    model.fit( target, **kwargs ) # Fit to the target, scikit-learn like

    plt.figure(figsize=(10,10))
    plot(target, "blue", alpha=.7)
    plot(source, "purple", alpha=.7)
    for t in [.5] : plot(model(t), "green", alpha=.4)
    plot(model(), "red")
    plt.axis("equal")
    plt.axis([0,1,0,1])
    
    plt.figure(figsize=(10,3))
    plt.subplot(1,3,1)
    plot(target, "blue", scale=.1)
    plot(source, "purple", scale=.1)
    plt.axis("equal")
    plt.axis([0,1,0,1])
    
    plt.subplot(1,3,2)
    plot(target, "blue", scale=.1)
    plot(model(.5), "green", scale=.1, alpha=.4)
    plt.axis("equal")
    plt.axis([0,1,0,1])
    
    plt.subplot(1,3,3)
    plot(target, "blue", scale=.1)
    plot(model(), "red", scale=.1)
    plt.axis("equal")
    plt.axis([0,1,0,1])
    
    plt.show()
    
    return model
